@@ -1,6 +1,6 @@
 /* Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
- * Copyright (c) 2007-2017, The Tor Project, Inc. */
+ * Copyright (c) 2007-2018, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -9,6 +9,10 @@
  **/
 
 #include "orconfig.h"
+#include "lib/crypt_ops/crypto_dh.h"
+#include "lib/crypt_ops/crypto_rand.h"
+
+#include "app/config/or_state_st.h"
 
 #include <stdio.h>
 #ifdef HAVE_FCNTL_H
@@ -22,46 +26,44 @@
 #include <dirent.h>
 #endif /* defined(_WIN32) */
 
+#include <math.h>
+
 /* These macros pull in declarations for some functions and structures that
  * are typically file-private. */
-#define GEOIP_PRIVATE
 #define ROUTER_PRIVATE
 #define CIRCUITSTATS_PRIVATE
 #define CIRCUITLIST_PRIVATE
 #define MAIN_PRIVATE
 #define STATEFILE_PRIVATE
 
-/*
- * Linux doesn't provide lround in math.h by default, but mac os does...
- * It's best just to leave math.h out of the picture entirely.
- */
-//#include <math.h>
-long int lround(double x);
-double fabs(double x);
+#include "core/or/or.h"
+#include "lib/err/backtrace.h"
+#include "lib/container/buffers.h"
+#include "core/or/circuitlist.h"
+#include "core/or/circuitstats.h"
+#include "lib/compress/compress.h"
+#include "app/config/config.h"
+#include "core/or/connection_edge.h"
+#include "feature/rend/rendcommon.h"
+#include "feature/rend/rendcache.h"
+#include "test/test.h"
+#include "core/mainloop/main.h"
+#include "lib/memarea/memarea.h"
+#include "core/crypto/onion.h"
+#include "core/crypto/onion_ntor.h"
+#include "core/crypto/onion_fast.h"
+#include "core/crypto/onion_tap.h"
+#include "core/or/policies.h"
+#include "feature/stats/rephist.h"
+#include "feature/nodelist/routerparse.h"
+#include "app/config/statefile.h"
+#include "lib/crypt_ops/crypto_curve25519.h"
 
-#include "or.h"
-#include "backtrace.h"
-#include "buffers.h"
-#include "circuitlist.h"
-#include "circuitstats.h"
-#include "compress.h"
-#include "config.h"
-#include "connection_edge.h"
-#include "geoip.h"
-#include "rendcommon.h"
-#include "rendcache.h"
-#include "test.h"
-#include "main.h"
-#include "memarea.h"
-#include "onion.h"
-#include "onion_ntor.h"
-#include "onion_fast.h"
-#include "onion_tap.h"
-#include "policies.h"
-#include "rephist.h"
-#include "routerparse.h"
-#include "statefile.h"
-#include "crypto_curve25519.h"
+#include "core/or/extend_info_st.h"
+#include "core/or/or_circuit_st.h"
+#include "feature/rend/rend_encoded_v2_service_descriptor_st.h"
+#include "feature/rend/rend_intro_point_st.h"
+#include "feature/rend/rend_service_descriptor_st.h"
 
 /** Run unit tests for the onion handshake code. */
 static void
@@ -143,7 +145,8 @@ test_bad_onion_handshake(void *arg)
   memset(junk_buf, 0, sizeof(junk_buf));
   crypto_pk_obsolete_public_hybrid_encrypt(pk,
                                junk_buf2, TAP_ONIONSKIN_CHALLENGE_LEN,
-                               junk_buf, DH_KEY_LEN, PK_PKCS1_OAEP_PADDING, 1);
+                               junk_buf, DH1024_KEY_LEN,
+                               PK_PKCS1_OAEP_PADDING, 1);
   tt_int_op(-1, OP_EQ,
             onion_skin_TAP_server_handshake(junk_buf2, pk, NULL,
                                             s_buf, s_keys, 40));
@@ -351,6 +354,18 @@ test_onion_queues(void *arg)
   tor_free(onionskin);
 }
 
+static crypto_cipher_t *crypto_rand_aes_cipher = NULL;
+
+// Mock replacement for crypto_rand: Generates bytes from a provided AES_CTR
+// cipher in <b>crypto_rand_aes_cipher</b>.
+static void
+crypto_rand_deterministic_aes(char *out, size_t n)
+{
+  tor_assert(crypto_rand_aes_cipher);
+  memset(out, 0, n);
+  crypto_cipher_crypt_inplace(crypto_rand_aes_cipher, out, n);
+}
+
 static void
 test_circuit_timeout(void *arg)
 {
@@ -380,6 +395,11 @@ test_circuit_timeout(void *arg)
 
   state = or_state_new();
 
+  // Use a deterministic RNG here, or else we'll get nondeterministic
+  // coverage in some of the circuitstats functions.
+  MOCK(crypto_rand, crypto_rand_deterministic_aes);
+  crypto_rand_aes_cipher = crypto_cipher_new("xyzzyplughplover");
+
   circuitbuild_running_unit_tests();
 #define timeout0 (build_time_t)(30*1000.0)
   initial.Xm = 3000;
@@ -514,6 +534,8 @@ test_circuit_timeout(void *arg)
   circuit_build_times_free_timeouts(&final);
   or_state_free(state);
   teardown_periodic_events();
+  UNMOCK(crypto_rand);
+  crypto_cipher_free(crypto_rand_aes_cipher);
 }
 
 /** Test encoding and parsing of rendezvous service descriptors. */
@@ -629,376 +651,6 @@ test_rend_fns(void *arg)
   tor_free(intro_points_encrypted);
 }
 
-  /* Record odd numbered fake-IPs using ipv6, even numbered fake-IPs
-   * using ipv4.  Since our fake geoip database is the same between
-   * ipv4 and ipv6, we should get the same result no matter which
-   * address family we pick for each IP. */
-#define SET_TEST_ADDRESS(i) do {                \
-    if ((i) & 1) {                              \
-      SET_TEST_IPV6(i);                         \
-      tor_addr_from_in6(&addr, &in6);           \
-    } else {                                    \
-      tor_addr_from_ipv4h(&addr, (uint32_t) i); \
-    }                                           \
-  } while (0)
-
-  /* Make sure that country ID actually works. */
-#define SET_TEST_IPV6(i) \
-  do {                                                          \
-    set_uint32(in6.s6_addr + 12, htonl((uint32_t) (i)));        \
-  } while (0)
-#define CHECK_COUNTRY(country, val) do {                                \
-    /* test ipv4 country lookup */                                      \
-    tt_str_op(country, OP_EQ,                                              \
-               geoip_get_country_name(geoip_get_country_by_ipv4(val))); \
-    /* test ipv6 country lookup */                                      \
-    SET_TEST_IPV6(val);                                                 \
-    tt_str_op(country, OP_EQ,                                              \
-               geoip_get_country_name(geoip_get_country_by_ipv6(&in6))); \
-  } while (0)
-
-/** Run unit tests for GeoIP code. */
-static void
-test_geoip(void *arg)
-{
-  int i, j;
-  time_t now = 1281533250; /* 2010-08-11 13:27:30 UTC */
-  char *s = NULL, *v = NULL;
-  const char *bridge_stats_1 =
-      "bridge-stats-end 2010-08-12 13:27:30 (86400 s)\n"
-      "bridge-ips zz=24,xy=8\n"
-      "bridge-ip-versions v4=16,v6=16\n"
-      "bridge-ip-transports <OR>=24\n",
-  *dirreq_stats_1 =
-      "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
-      "dirreq-v3-ips ab=8\n"
-      "dirreq-v3-reqs ab=8\n"
-      "dirreq-v3-resp ok=0,not-enough-sigs=0,unavailable=0,not-found=0,"
-          "not-modified=0,busy=0\n"
-      "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
-      "dirreq-v3-tunneled-dl complete=0,timeout=0,running=0\n",
-  *dirreq_stats_2 =
-      "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
-      "dirreq-v3-ips \n"
-      "dirreq-v3-reqs \n"
-      "dirreq-v3-resp ok=0,not-enough-sigs=0,unavailable=0,not-found=0,"
-          "not-modified=0,busy=0\n"
-      "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
-      "dirreq-v3-tunneled-dl complete=0,timeout=0,running=0\n",
-  *dirreq_stats_3 =
-      "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
-      "dirreq-v3-ips \n"
-      "dirreq-v3-reqs \n"
-      "dirreq-v3-resp ok=8,not-enough-sigs=0,unavailable=0,not-found=0,"
-          "not-modified=0,busy=0\n"
-      "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
-      "dirreq-v3-tunneled-dl complete=0,timeout=0,running=0\n",
-  *dirreq_stats_4 =
-      "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
-      "dirreq-v3-ips \n"
-      "dirreq-v3-reqs \n"
-      "dirreq-v3-resp ok=8,not-enough-sigs=0,unavailable=0,not-found=0,"
-          "not-modified=0,busy=0\n"
-      "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
-      "dirreq-v3-tunneled-dl complete=0,timeout=0,running=4\n",
-  *entry_stats_1 =
-      "entry-stats-end 2010-08-12 13:27:30 (86400 s)\n"
-      "entry-ips ab=8\n",
-  *entry_stats_2 =
-      "entry-stats-end 2010-08-12 13:27:30 (86400 s)\n"
-      "entry-ips \n";
-  tor_addr_t addr;
-  struct in6_addr in6;
-
-  /* Populate the DB a bit.  Add these in order, since we can't do the final
-   * 'sort' step.  These aren't very good IP addresses, but they're perfectly
-   * fine uint32_t values. */
-  (void)arg;
-  tt_int_op(0,OP_EQ, geoip_parse_entry("10,50,AB", AF_INET));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("52,90,XY", AF_INET));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("95,100,AB", AF_INET));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("\"105\",\"140\",\"ZZ\"", AF_INET));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("\"150\",\"190\",\"XY\"", AF_INET));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("\"200\",\"250\",\"AB\"", AF_INET));
-
-  /* Populate the IPv6 DB equivalently with fake IPs in the same range */
-  tt_int_op(0,OP_EQ, geoip_parse_entry("::a,::32,AB", AF_INET6));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("::34,::5a,XY", AF_INET6));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("::5f,::64,AB", AF_INET6));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("::69,::8c,ZZ", AF_INET6));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("::96,::be,XY", AF_INET6));
-  tt_int_op(0,OP_EQ, geoip_parse_entry("::c8,::fa,AB", AF_INET6));
-
-  /* We should have 4 countries: ??, ab, xy, zz. */
-  tt_int_op(4,OP_EQ, geoip_get_n_countries());
-  memset(&in6, 0, sizeof(in6));
-
-  CHECK_COUNTRY("??", 3);
-  CHECK_COUNTRY("ab", 32);
-  CHECK_COUNTRY("??", 5);
-  CHECK_COUNTRY("??", 51);
-  CHECK_COUNTRY("xy", 150);
-  CHECK_COUNTRY("xy", 190);
-  CHECK_COUNTRY("??", 2000);
-
-  tt_int_op(0,OP_EQ, geoip_get_country_by_ipv4(3));
-  SET_TEST_IPV6(3);
-  tt_int_op(0,OP_EQ, geoip_get_country_by_ipv6(&in6));
-
-  get_options_mutable()->BridgeRelay = 1;
-  get_options_mutable()->BridgeRecordUsageByCountry = 1;
-  /* Put 9 observations in AB... */
-  for (i=32; i < 40; ++i) {
-    SET_TEST_ADDRESS(i);
-    geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now-7200);
-  }
-  SET_TEST_ADDRESS(225);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now-7200);
-  /* and 3 observations in XY, several times. */
-  for (j=0; j < 10; ++j)
-    for (i=52; i < 55; ++i) {
-      SET_TEST_ADDRESS(i);
-      geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now-3600);
-    }
-  /* and 17 observations in ZZ... */
-  for (i=110; i < 127; ++i) {
-    SET_TEST_ADDRESS(i);
-    geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now);
-  }
-  geoip_get_client_history(GEOIP_CLIENT_CONNECT, &s, &v);
-  tt_assert(s);
-  tt_assert(v);
-  tt_str_op("zz=24,ab=16,xy=8",OP_EQ, s);
-  tt_str_op("v4=16,v6=16",OP_EQ, v);
-  tor_free(s);
-  tor_free(v);
-
-  /* Now clear out all the AB observations. */
-  geoip_remove_old_clients(now-6000);
-  geoip_get_client_history(GEOIP_CLIENT_CONNECT, &s, &v);
-  tt_assert(s);
-  tt_assert(v);
-  tt_str_op("zz=24,xy=8",OP_EQ, s);
-  tt_str_op("v4=16,v6=16",OP_EQ, v);
-  tor_free(s);
-  tor_free(v);
-
-  /* Start testing bridge statistics by making sure that we don't output
-   * bridge stats without initializing them. */
-  s = geoip_format_bridge_stats(now + 86400);
-  tt_ptr_op(s, OP_EQ, NULL);
-
-  /* Initialize stats and generate the bridge-stats history string out of
-   * the connecting clients added above. */
-  geoip_bridge_stats_init(now);
-  s = geoip_format_bridge_stats(now + 86400);
-  tt_assert(s);
-  tt_str_op(bridge_stats_1,OP_EQ, s);
-  tor_free(s);
-
-  /* Stop collecting bridge stats and make sure we don't write a history
-   * string anymore. */
-  geoip_bridge_stats_term();
-  s = geoip_format_bridge_stats(now + 86400);
-  tt_ptr_op(s, OP_EQ, NULL);
-
-  /* Stop being a bridge and start being a directory mirror that gathers
-   * directory request statistics. */
-  geoip_bridge_stats_term();
-  get_options_mutable()->BridgeRelay = 0;
-  get_options_mutable()->BridgeRecordUsageByCountry = 0;
-  get_options_mutable()->DirReqStatistics = 1;
-
-  /* Start testing dirreq statistics by making sure that we don't collect
-   * dirreq stats without initializing them. */
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_NETWORKSTATUS, &addr, NULL, now);
-  s = geoip_format_dirreq_stats(now + 86400);
-  tt_ptr_op(s, OP_EQ, NULL);
-
-  /* Initialize stats, note one connecting client, and generate the
-   * dirreq-stats history string. */
-  geoip_dirreq_stats_init(now);
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_NETWORKSTATUS, &addr, NULL, now);
-  s = geoip_format_dirreq_stats(now + 86400);
-  tt_str_op(dirreq_stats_1,OP_EQ, s);
-  tor_free(s);
-
-  /* Stop collecting stats, add another connecting client, and ensure we
-   * don't generate a history string. */
-  geoip_dirreq_stats_term();
-  SET_TEST_ADDRESS(101);
-  geoip_note_client_seen(GEOIP_CLIENT_NETWORKSTATUS, &addr, NULL, now);
-  s = geoip_format_dirreq_stats(now + 86400);
-  tt_ptr_op(s, OP_EQ, NULL);
-
-  /* Re-start stats, add a connecting client, reset stats, and make sure
-   * that we get an all empty history string. */
-  geoip_dirreq_stats_init(now);
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_NETWORKSTATUS, &addr, NULL, now);
-  geoip_reset_dirreq_stats(now);
-  s = geoip_format_dirreq_stats(now + 86400);
-  tt_str_op(dirreq_stats_2,OP_EQ, s);
-  tor_free(s);
-
-  /* Note a successful network status response and make sure that it
-   * appears in the history string. */
-  geoip_note_ns_response(GEOIP_SUCCESS);
-  s = geoip_format_dirreq_stats(now + 86400);
-  tt_str_op(dirreq_stats_3,OP_EQ, s);
-  tor_free(s);
-
-  /* Start a tunneled directory request. */
-  geoip_start_dirreq((uint64_t) 1, 1024, DIRREQ_TUNNELED);
-  s = geoip_format_dirreq_stats(now + 86400);
-  tt_str_op(dirreq_stats_4,OP_EQ, s);
-  tor_free(s);
-
-  /* Stop collecting directory request statistics and start gathering
-   * entry stats. */
-  geoip_dirreq_stats_term();
-  get_options_mutable()->DirReqStatistics = 0;
-  get_options_mutable()->EntryStatistics = 1;
-
-  /* Start testing entry statistics by making sure that we don't collect
-   * anything without initializing entry stats. */
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now);
-  s = geoip_format_entry_stats(now + 86400);
-  tt_ptr_op(s, OP_EQ, NULL);
-
-  /* Initialize stats, note one connecting client, and generate the
-   * entry-stats history string. */
-  geoip_entry_stats_init(now);
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now);
-  s = geoip_format_entry_stats(now + 86400);
-  tt_str_op(entry_stats_1,OP_EQ, s);
-  tor_free(s);
-
-  /* Stop collecting stats, add another connecting client, and ensure we
-   * don't generate a history string. */
-  geoip_entry_stats_term();
-  SET_TEST_ADDRESS(101);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now);
-  s = geoip_format_entry_stats(now + 86400);
-  tt_ptr_op(s, OP_EQ, NULL);
-
-  /* Re-start stats, add a connecting client, reset stats, and make sure
-   * that we get an all empty history string. */
-  geoip_entry_stats_init(now);
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now);
-  geoip_reset_entry_stats(now);
-  s = geoip_format_entry_stats(now + 86400);
-  tt_str_op(entry_stats_2,OP_EQ, s);
-  tor_free(s);
-
-  /* Test the OOM handler. Add a client, run the OOM. */
-  geoip_entry_stats_init(now);
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL,
-                         now - (12 * 60 * 60));
-  /* We've seen this 12 hours ago. Run the OOM, it should clean the entry
-   * because it is above the minimum cutoff of 4 hours. */
-  size_t bytes_removed = geoip_client_cache_handle_oom(now, 1000);
-  tt_size_op(bytes_removed, OP_GT, 0);
-
-  /* Do it again but this time with an entry with a lower cutoff. */
-  geoip_entry_stats_init(now);
-  SET_TEST_ADDRESS(100);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL,
-                         now - (3 * 60 * 60));
-  bytes_removed = geoip_client_cache_handle_oom(now, 1000);
-  tt_size_op(bytes_removed, OP_EQ, 0);
-
-  /* Stop collecting entry statistics. */
-  geoip_entry_stats_term();
-  get_options_mutable()->EntryStatistics = 0;
-
- done:
-  tor_free(s);
-  tor_free(v);
-}
-
-static void
-test_geoip_with_pt(void *arg)
-{
-  time_t now = 1281533250; /* 2010-08-11 13:27:30 UTC */
-  char *s = NULL;
-  int i;
-  tor_addr_t addr;
-  struct in6_addr in6;
-
-  (void)arg;
-  get_options_mutable()->BridgeRelay = 1;
-  get_options_mutable()->BridgeRecordUsageByCountry = 1;
-
-  memset(&in6, 0, sizeof(in6));
-
-  /* No clients seen yet. */
-  s = geoip_get_transport_history();
-  tor_assert(!s);
-
-  /* 4 connections without a pluggable transport */
-  for (i=0; i < 4; ++i) {
-    SET_TEST_ADDRESS(i);
-    geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, NULL, now-7200);
-  }
-
-  /* 9 connections with "alpha" */
-  for (i=4; i < 13; ++i) {
-    SET_TEST_ADDRESS(i);
-    geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, "alpha", now-7200);
-  }
-
-  /* one connection with "beta" */
-  SET_TEST_ADDRESS(13);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, "beta", now-7200);
-
-  /* 14 connections with "charlie" */
-  for (i=14; i < 28; ++i) {
-    SET_TEST_ADDRESS(i);
-    geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, "charlie", now-7200);
-  }
-
-  /* 131 connections with "ddr" */
-  for (i=28; i < 159; ++i) {
-    SET_TEST_ADDRESS(i);
-    geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, "ddr", now-7200);
-  }
-
-  /* 8 connections with "entropy" */
-  for (i=159; i < 167; ++i) {
-    SET_TEST_ADDRESS(i);
-    geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, "entropy", now-7200);
-  }
-
-  /* 2 connections from the same IP with two different transports. */
-  SET_TEST_ADDRESS(++i);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, "fire", now-7200);
-  geoip_note_client_seen(GEOIP_CLIENT_CONNECT, &addr, "google", now-7200);
-
-  /* Test the transport history string. */
-  s = geoip_get_transport_history();
-  tor_assert(s);
-  tt_str_op(s,OP_EQ, "<OR>=8,alpha=16,beta=8,charlie=16,ddr=136,"
-             "entropy=8,fire=8,google=8");
-
-  /* Stop collecting entry statistics. */
-  geoip_entry_stats_term();
-  get_options_mutable()->EntryStatistics = 0;
-
- done:
-  tor_free(s);
-}
-
-#undef SET_TEST_ADDRESS
-#undef SET_TEST_IPV6
-#undef CHECK_COUNTRY
-
 /** Run unit tests for stats code. */
 static void
 test_stats(void *arg)
@@ -1172,8 +824,6 @@ static struct testcase_t test_array[] = {
   { "fast_handshake", test_fast_handshake, 0, NULL, NULL },
   FORK(circuit_timeout),
   FORK(rend_fns),
-  ENT(geoip),
-  FORK(geoip_with_pt),
   FORK(stats),
 
   END_OF_TESTCASES
@@ -1185,7 +835,9 @@ struct testgroup_t testgroups[] = {
   { "addr/", addr_tests },
   { "address/", address_tests },
   { "address_set/", address_set_tests },
+  { "bridges/", bridges_tests },
   { "buffer/", buffer_tests },
+  { "bwmgt/", bwmgt_tests },
   { "cellfmt/", cell_format_tests },
   { "cellqueue/", cell_queue_tests },
   { "channel/", channel_tests },
@@ -1207,15 +859,21 @@ struct testgroup_t testgroups[] = {
   { "control/", controller_tests },
   { "control/event/", controller_event_tests },
   { "crypto/", crypto_tests },
+  { "crypto/ope/", crypto_ope_tests },
+#ifdef ENABLE_OPENSSL
   { "crypto/openssl/", crypto_openssl_tests },
+#endif
+  { "crypto/pem/", pem_tests },
   { "dir/", dir_tests },
   { "dir_handle_get/", dir_handle_get_tests },
   { "dir/md/", microdesc_tests },
+  { "dir/voting-schedule/", voting_schedule_tests },
   { "dos/", dos_tests },
   { "entryconn/", entryconn_tests },
   { "entrynodes/", entrynodes_tests },
   { "guardfraction/", guardfraction_tests },
   { "extorport/", extorport_tests },
+  { "geoip/", geoip_tests },
   { "legacy_hs/", hs_tests },
   { "hs_cache/", hs_cache },
   { "hs_cell/", hs_cell_tests },
@@ -1230,10 +888,12 @@ struct testgroup_t testgroups[] = {
   { "introduce/", introduce_tests },
   { "keypin/", keypin_tests },
   { "link-handshake/", link_handshake_tests },
+  { "mainloop/", mainloop_tests },
   { "nodelist/", nodelist_tests },
   { "oom/", oom_tests },
   { "oos/", oos_tests },
   { "options/", options_tests },
+  { "periodic-event/" , periodic_event_tests },
   { "policy/" , policy_tests },
   { "procmon/", procmon_tests },
   { "proto/http/", proto_http_tests },
@@ -1242,6 +902,7 @@ struct testgroup_t testgroups[] = {
   { "pt/", pt_tests },
   { "relay/" , relay_tests },
   { "relaycell/", relaycell_tests },
+  { "relaycrypt/", relaycrypt_tests },
   { "rend_cache/", rend_cache_tests },
   { "replaycache/", replaycache_tests },
   { "router/", router_tests },
@@ -1254,14 +915,16 @@ struct testgroup_t testgroups[] = {
   { "status/" , status_tests },
   { "storagedir/", storagedir_tests },
   { "tortls/", tortls_tests },
+#ifndef ENABLE_NSS
+  { "tortls/openssl/", tortls_openssl_tests },
+#endif
+  { "tortls/x509/", x509_tests },
   { "util/", util_tests },
   { "util/format/", util_format_tests },
   { "util/logging/", logging_tests },
   { "util/process/", util_process_tests },
-  { "util/pubsub/", pubsub_tests },
   { "util/thread/", thread_tests },
   { "util/handle/", handle_tests },
   { "dns/", dns_tests },
   END_OF_GROUPS
 };
-
